@@ -33,6 +33,8 @@ from trollius import Return, From
 from trollius import futures
 
 import trollius as asyncio
+from trollius import compat
+from trollius import events
 from trollius import proactor_events
 from trollius import selector_events
 from trollius import test_utils
@@ -619,15 +621,48 @@ class EventLoopTestsMixin(object):
         self.assertGreater(pr.nbytes, 0)
         tr.close()
 
+    def _test_create_ssl_connection(self, httpd, create_connection,
+                                    check_sockname=True):
+        conn_fut = create_connection(ssl=test_utils.dummy_ssl_context())
+        self._basetest_create_ssl_connection(conn_fut, check_sockname)
+
+        # ssl.Purpose was introduced in Python 3.4
+        #if not asyncio.BACKPORT_SSL_CONTEXT:
+        if hasattr(ssl, 'Purpose'):
+            def _dummy_ssl_create_context(purpose=ssl.Purpose.SERVER_AUTH,
+                                          cafile=None, capath=None,
+                                          cadata=None):
+                """
+                A ssl.create_default_context() replacement that doesn't enable
+                cert validation.
+                """
+                self.assertEqual(purpose, ssl.Purpose.SERVER_AUTH)
+                return test_utils.dummy_ssl_context()
+
+            # With ssl=True, ssl.create_default_context() should be called
+            with mock.patch('ssl.create_default_context',
+                            side_effect=_dummy_ssl_create_context) as m:
+                conn_fut = create_connection(ssl=True)
+                self._basetest_create_ssl_connection(conn_fut, check_sockname)
+                self.assertEqual(m.call_count, 1)
+
+        if not asyncio.BACKPORT_SSL_CONTEXT:
+            # With the real ssl.create_default_context(), certificate
+            # validation will fail
+            with self.assertRaises(ssl.SSLError) as cm:
+                conn_fut = create_connection(ssl=True)
+                # Ignore the "SSL handshake failed" log in debug mode
+                with test_utils.disable_logger():
+                    self._basetest_create_ssl_connection(conn_fut, check_sockname)
+
     @test_utils.skipIf(ssl is None, 'No ssl module')
     def test_create_ssl_connection(self):
         with test_utils.run_test_server(use_ssl=True) as httpd:
-            conn_fut = self.loop.create_connection(
+            create_connection = functools.partial(
+                self.loop.create_connection,
                 lambda: MyProto(loop=self.loop),
-                *httpd.address,
-                ssl=test_utils.dummy_ssl_context())
-
-            self._basetest_create_ssl_connection(conn_fut)
+                *httpd.address)
+            self._test_create_ssl_connection(httpd, create_connection)
 
     @test_utils.skipIf(ssl is None, 'No ssl module')
     @test_utils.skipUnless(hasattr(socket, 'AF_UNIX'), 'No UNIX Sockets')
@@ -637,13 +672,13 @@ class EventLoopTestsMixin(object):
         check_sockname = not osx_tiger()
 
         with test_utils.run_test_unix_server(use_ssl=True) as httpd:
-            conn_fut = self.loop.create_unix_connection(
-                lambda: MyProto(loop=self.loop),
-                httpd.address,
-                ssl=test_utils.dummy_ssl_context(),
+            create_connection = functools.partial(
+                self.loop.create_unix_connection,
+                lambda: MyProto(loop=self.loop), httpd.address,
                 server_hostname='127.0.0.1')
 
-            self._basetest_create_ssl_connection(conn_fut, check_sockname)
+            self._test_create_ssl_connection(httpd, create_connection,
+                                             check_sockname)
 
     def test_create_connection_local_addr(self):
         with test_utils.run_test_server() as httpd:
@@ -910,13 +945,19 @@ class EventLoopTestsMixin(object):
         if hasattr(sslcontext_client, 'check_hostname'):
             sslcontext_client.check_hostname = True
 
+        if compat.PY3:
+            err_msg = "hostname '127.0.0.1' doesn't match 'localhost'"
+        else:
+            # http://bugs.python.org/issue22861
+            err_msg = "hostname '127.0.0.1' doesn't match u'localhost'"
+
         # incorrect server_hostname
         f_c = self.loop.create_connection(MyProto, host, port,
                                           ssl=sslcontext_client)
         with test_utils.disable_logger():
             with self.assertRaisesRegex(
                     ssl.CertificateError,
-                    "hostname '127.0.0.1' doesn't match 'localhost'"):
+                    err_msg):
                 self.loop.run_until_complete(f_c)
 
         # close connection
@@ -2075,12 +2116,15 @@ class AbstractEventLoopTests(test_utils.TestCase):
             NotImplementedError, loop.stop)
         self.assertRaises(
             NotImplementedError, loop.is_running)
-        self.assertRaises(
-            NotImplementedError, loop.is_closed)
+        # skip some tests if the AbstractEventLoop class comes from asyncio
+        # and the asyncio version (python version in fact) is older than 3.4.2
+        if events.asyncio is None or sys.version_info >= (3, 4, 2):
+            self.assertRaises(
+                NotImplementedError, loop.is_closed)
+            self.assertRaises(
+                NotImplementedError, loop.create_task, None)
         self.assertRaises(
             NotImplementedError, loop.close)
-        self.assertRaises(
-            NotImplementedError, loop.create_task, None)
         self.assertRaises(
             NotImplementedError, loop.call_later, None, None)
         self.assertRaises(
