@@ -273,7 +273,15 @@ class BaseEventLoop(events.AbstractEventLoop):
             future._log_destroy_pending = False
 
         future.add_done_callback(_raise_stop_error)
-        self.run_forever()
+        try:
+            self.run_forever()
+        except:
+            if new_task and future.done() and not future.cancelled():
+                # The coroutine raised a BaseException. Consume the exception
+                # to not log a warning, the caller doesn't have access to the
+                # local task.
+                future.exception()
+            raise
         future.remove_done_callback(_raise_stop_error)
         if not future.done():
             raise RuntimeError('Event loop stopped before Future completed.')
@@ -354,7 +362,8 @@ class BaseEventLoop(events.AbstractEventLoop):
 
         Absolute time corresponds to the event loop's time() method.
         """
-        if coroutines.iscoroutinefunction(callback):
+        if (coroutines.iscoroutine(callback)
+        or coroutines.iscoroutinefunction(callback)):
             raise TypeError("coroutines cannot be used with call_at()")
         if self._debug:
             self._assert_is_current_event_loop()
@@ -381,7 +390,8 @@ class BaseEventLoop(events.AbstractEventLoop):
         return handle
 
     def _call_soon(self, callback, args, check_loop):
-        if coroutines.iscoroutinefunction(callback):
+        if (coroutines.iscoroutine(callback)
+        or coroutines.iscoroutinefunction(callback)):
             raise TypeError("coroutines cannot be used with call_soon()")
         if self._debug and check_loop:
             self._assert_is_current_event_loop()
@@ -418,8 +428,9 @@ class BaseEventLoop(events.AbstractEventLoop):
         return handle
 
     def run_in_executor(self, executor, callback, *args):
-        if coroutines.iscoroutinefunction(callback):
-            raise TypeError("Coroutines cannot be used with run_in_executor()")
+        if (coroutines.iscoroutine(callback)
+        or coroutines.iscoroutinefunction(callback)):
+            raise TypeError("coroutines cannot be used with run_in_executor()")
         if isinstance(callback, events.Handle):
             assert not args
             assert not isinstance(callback, events.TimerHandle)
@@ -1010,19 +1021,22 @@ class BaseEventLoop(events.AbstractEventLoop):
         'call_later' callbacks.
         """
 
-        # Remove delayed calls that were cancelled if their number is too high
         sched_count = len(self._scheduled)
         if (sched_count > _MIN_SCHEDULED_TIMER_HANDLES and
             float(self._timer_cancelled_count) / sched_count >
                 _MIN_CANCELLED_TIMER_HANDLES_FRACTION):
+            # Remove delayed calls that were cancelled if their number
+            # is too high
+            new_scheduled = []
             for handle in self._scheduled:
                 if handle._cancelled:
                     handle._scheduled = False
+                else:
+                    new_scheduled.append(handle)
 
-            self._scheduled = [x for x in self._scheduled if not x._cancelled]
+            heapq.heapify(new_scheduled)
+            self._scheduled = new_scheduled
             self._timer_cancelled_count = 0
-
-            heapq.heapify(self._scheduled)
         else:
             # Remove delayed calls that were cancelled from head of queue.
             while self._scheduled and self._scheduled[0]._cancelled:
