@@ -63,7 +63,7 @@ class Lock(object):
 
     acquire() is a coroutine and should be called with 'yield From'.
 
-    Locks also support the context manager protocol.  '(yield From(lock))'
+    Locks also support the context management protocol.  '(yield From(lock))'
     should be used as context manager expression.
 
     Usage:
@@ -239,14 +239,17 @@ class Condition(object):
     A new Lock object is created and used as the underlying lock.
     """
 
-    def __init__(self, loop=None):
+    def __init__(self, lock=None, loop=None):
         if loop is not None:
             self._loop = loop
         else:
             self._loop = events.get_event_loop()
 
-        # Lock as an attribute as in threading.Condition.
-        lock = Lock(loop=self._loop)
+        if lock is None:
+            lock = Lock(loop=self._loop)
+        elif lock._loop is not self._loop:
+            raise ValueError("loop argument must agree with lock")
+
         self._lock = lock
         # Export the lock's locked(), acquire() and release() methods.
         self.locked = lock.locked
@@ -352,7 +355,7 @@ class Semaphore(object):
     can never go below zero; when acquire() finds that it is zero, it blocks,
     waiting until some other thread calls release().
 
-    Semaphores also support the context manager protocol.
+    Semaphores also support the context management protocol.
 
     The optional argument gives the initial value for the internal
     counter; it defaults to 1. If the value given is less than 0,
